<?php

namespace App\Http\Controllers\Admin;

use App\Exports\MagzExport;
use App\Helpers\{AnalyticHelper};
use App\Http\Controllers\Controller;
use App\Imports\MagzImport;
use App\Models\{Language, User};
use App\Services\SettingService;
use GeoSot\EnvEditor\Facades\EnvEditor;
use Illuminate\Contracts\Foundation\Application;
use Illuminate\Contracts\View\{Factory, View};
use Illuminate\Http\{JsonResponse, Request};
use Illuminate\Support\{Arr, Str};
use Illuminate\Support\Facades\{App, Auth, Cache, DB, File, Redirect, Session, Validator, Storage};
use Illuminate\Validation\Rule;
use Intervention\Image\Facades\Image;
use Maatwebsite\Excel\Facades\Excel;
use Mcamara\LaravelLocalization\Facades\LaravelLocalization;
use Spatie\Analytics\Period;
use ZipArchive;

class SettingController extends Controller
{
    private $settingService;
    
    /**
     * __construct
     *
     * @param  mixed $settingService
     * @return void
     */
    public function __construct(SettingService $settingService)
    {
        $this->settingService = $settingService;

        $this->middleware('permission:read-settings', ['only' => ['index', 'setting']]);
        $this->middleware('permission:update-settings', [
            'only' => ['updateSettings', 'changeMaintenance', 'changeRegisterMember', 'changeActiveEmailVerification', 'settingUpdate']
        ]);
    }

    /**
     * index
     *
     * @return void
     */
    public function index()
    {
        $dir               = config('settings.current_theme_dir');
        $maintenance       = config('settings.maintenance') === 'y' ? 'checked' : '';
        $register          = config('settings.register') === 'y' ? 'checked' : '';
        $emailVerification = config('settings.email_verification') === 'y' ? 'checked' : '';
        $showLanguage      = config('settings.display_language') === 'y' ? 'checked' : '';
        $links             = json_decode(config('settings.links'));

        $creditFooter = File::get(resource_path('views/frontend/' . $dir . '/inc/_credit-footer.blade.php'));

        $languages = Language::select('name', 'language', 'active')->where('active', 'y')->pluck('name', 'language');

        $propertyId =  EnvEditor::getKey("ANALYTICS_PROPERTY_ID");

        $logoWebLight  = $this->settingService->getAssetImage(config('settings.logo_web_light'));
        $logoWebDark   = $this->settingService->getAssetImage(config('settings.logo_web_dark'));
        $favicon       = $this->settingService->getAssetImage(config('settings.favicon'));
        $ogiHomepage   = $this->settingService->getAssetImage(config('settings.ogi_homepage'));
        $logoDashboard = $this->settingService->getAssetImage(config('settings.logo_dashboard'));
        $logoAuth      = $this->settingService->getAssetImage(config('settings.logo_auth'));

        return view('admin.settings.index', compact(
            'propertyId',
            'creditFooter',
            'favicon',
            'languages',
            'logoAuth',
            'logoDashboard',
            'logoWebLight',
            'logoWebDark',
            'maintenance',
            'ogiHomepage',
            'register',
            'emailVerification',
            'showLanguage',
            'links'
        ));
    }

    /**
     * @return Application|SessionManager|Store|mixed
     */
    public function sessionDeviceSetPeriode()
    {
        $period  = (int)request('periode');
        $devices = AnalyticHelper::getTopDevice(Period::days((int)request('periode')))->toArray();
        $num_col = $devices ? count($devices) : 12;
        $col     = 12 / $num_col;

        return [
            'period' => AnalyticHelper::getLabel($period),
            'data'   => $devices,
            'col'    => $col
        ];
    }

    /**
     * @return Application|SessionManager|Store|mixed
     */
    public function visitorPageViewSetPeriode()
    {
        $period  = request('periode');

        $totalVP  = AnalyticHelper::fetchTotalVisitorsAndPageViews(Period::days($period));
        $dates_VP = $totalVP->map(function ($item) {
            return $item['date']->format('d M');
        })->toArray();

        $visitors_VP    = $totalVP->pluck('activeUsers');
        $pageViews_VP   = $totalVP->pluck('screenPageViews');
        $totalPageViews = $totalVP->sum('screenPageViews');
        $totalVisitors  = $totalVP->sum('activeUsers');

        $chart = [
            'chart' => [
                'labels' => Arr::flatten($dates_VP),
            ],
            'datasets' => [
                [
                    'name' => 'Visitors',
                    'values' => Arr::flatten($visitors_VP),
                ],
                [
                    'name' => 'PageViews',
                    'values' => Arr::flatten($pageViews_VP),
                ]
            ],
            'period' => AnalyticHelper::getLabel($period),
            'total' => [
                'totalPageViews' => $totalPageViews,
                'totalVisitors' => $totalVisitors
            ]
        ];

        return $chart;
    }

    /**
     * @return Application|SessionManager|Store|mixed
     */
    public function mostVisitedPages()
    {
        return session(['session_most_visited_pages' => (int)request('periode')]);
    }

    /**
     * @return Application|SessionManager|Store|mixed
     */
    public function browserUsed()
    {
        return session(['session_browser_used' => (int)request('periode')]);
    }

    /**
     * @return Application|SessionManager|Store|mixed
     */
    public function operatingSystem()
    {
        return session(['session_browser_used' => (int)request('periode')]);
    }

    /**
     * @return Application|SessionManager|Store|mixed
     */
    public function sessionsCountry()
    {
        return session(['session_country' => (int)request('periode')]);
    }

    /**
     * @param Request $request
     * @return mixed
     */
    public function changeDashboardLanguage(Request $request)
    {
        Cache::forget('settings');
        return User::find(Auth::id())->update(['language' => request('id')]);
    }

    /**
     * @param $lang
     * @return \Illuminate\Http\RedirectResponse
     */
    public function changeCurrentUserLanguage($lang)
    {
        Cache::forget('settings');
        User::find(Auth::id())->update(['language' => $lang]);
        return Redirect::back();
    }

    /**
     * @param Request $request
     * @return JsonResponse
     */
    public function changeDefaultLanguage(Request $request): JsonResponse
    {
        Cache::forget('settings');
        App::setLocale(request('code'));
        Session::put('locale', request('code'));
        LaravelLocalization::setLocale(request('code'));

        $this->settingService->modify('default_language', request('code'));

        return response()->json(['success' => __('message.saved_successfully')]);
    }

    /**
     * @param Request $request
     * @return JsonResponse
     */
    public function changeMaintenance(Request $request)
    {
        Cache::forget('settings');

        $this->settingService->modify('maintenance', request('active'));

        $msg = request('active') === 'y' ?
            __('message.mode_maintenance_enabled') : __('message.mode_maintenance_disabled');

        return response()->json(['success' => $msg]);
    }
    
    /**
     * setConfig
     *
     * @param  mixed $request
     * @return void
     */
    public function setConfig(Request $request)
    {
        Cache::forget('settings');

        if ($request->key == 'permalink_type') {
            if ($request->value == 'custom') {
                $this->settingService->modify('permalink', $request->custom);
                $this->settingService->modify('permalink_old_custom', $request->custom);
            } else {
                $this->settingService->modify('permalink', $request->value);
                $this->settingService->modify('permalink_old_custom', config('settings.permalink_old_custom'));
            }
        }

        $this->settingService->modify($request->key, $request->value);

        $msg =  __('message.changed_successfully');

        return response()->json(['success' => $msg]);
    }

    /**
     * @param Request $request
     * @return JsonResponse
     */
    public function changeRegisterMember(Request $request)
    {
        Cache::forget('settings');

        
        $currentUser = Auth::User();

        if(!$currentUser->hasRole(['super-admin', 'admin'])) {
            return response()->json(['abort' => __('message.unauthorized_action')]);
        }

        $this->settingService->modify('register', request('active'));

        $msg = request('active') === 'y' ?
            __('message.register_enabled') : __('message.register_disabled');

        return response()->json(['success' => $msg]);
    }

    /**
     * @param Request $request
     * @return JsonResponse
     */
    public function changeActiveEmailVerification(Request $request)
    {
        Cache::forget('settings');
        if(!Auth::User()->hasRole(['super-admin', 'admin'])) {
            return response()->json(['abort' => __('message.unauthorized_action')]);
        }

        $this->settingService->modify('email_verification', request('active'));
        
        if (request('active') === 'y') {
            $msg = __('message.email_verification_enabled');
        } else {
            $msg = __('message.email_verification_disabled');
        }

        return response()->json(['success' => $msg]);
    }

    /**
     * @param Request $request
     * @return JsonResponse
     */
    public function changeDisplayLanguage(Request $request): JsonResponse
    {
        Cache::forget('settings');
        $this->settingService->modify('display_language', request('active'));

        $msg = request('active') === 'y' ?
            __('setting.display_language_enabled') : __('message.display_language_disabled');

        return response()->json(['success' => $msg]);
    }

    /**
     * @param Request $request
     * @return JsonResponse|\Illuminate\Http\RedirectResponse|void
     * @throws \Illuminate\Auth\Access\AuthorizationException
     */
    public function settingUpdate(Request $request)
    {
        $this->authorize('update-settings');

        $validator = Validator::make($request->all(), [
            'company_name' => Rule::requiredIf(request()->has('web_information')),
            'site_name'    => Rule::requiredIf(request()->has('web_information')),
            'site_url'     => Rule::requiredIf(request()->has('web_information')),
            'favicon'      => 'dimensions:max_width=256,max_height=256|mimes:jpeg,png,ico',
            'logo_website' => 'dimensions:max_width=800,max_height=800|image|mimes:jpeg,png',
            'og_image'     => 'dimensions:max_width=1484,max_height=1200|image|mimes:jpeg,png'
        ]);

        if (request()->has('site_logo')) {
            if ($validator->fails()) {
                return redirect()->route('settings.index')->with('error', $validator->errors()->first());
            }
        } else {
            if ($validator->fails()) {
                return response()->json(['errors' => $validator->errors()->all()]);
            }
        }

        if (request()->has('web_information')) {

            if (request('credit_footer')) {
                $dir = config('settings.current_theme_dir');
                File::put(resource_path('views/frontend/'.$dir.'/inc/_credit-footer.blade.php'), request('credit_footer'));
            }

            $arrayValues = [
                ['key' => 'company_name', 'value' => request('company_name')],
                ['key' => 'site_name', 'value' => request('site_name')],
                ['key' => 'site_url', 'value' => request('site_url')],
                ['key' => 'site_description', 'value' => request('site_description')],
                ['key' => 'meta_keyword', 'value' => request('meta_keyword')],
            ];

            Cache::forget('settings');

            foreach ($arrayValues as $value) {
                $this->settingService->modify($value['key'], $value['value']);
            }

            return response()->json(['success' => __('message.saved_successfully')]);
        }

        if (request()->has('web_contact')) {
            $arrayValues = [
                ['key' => 'street', 'value' => request('street')],
                ['key' => 'city', 'value' => request('city')],
                ['key' => 'postal_code', 'value' => request('postal_code')],
                ['key' => 'state', 'value' => request('state')],
                ['key' => 'country', 'value' => request('country')],
                ['key' => 'site_phone', 'value' => request('site_phone')],
                ['key' => 'site_email', 'value' => request('site_email')],
                ['key' => 'contact_description', 'value' => request('contact_description')]
            ];

            Cache::forget('settings');

            foreach ($arrayValues as $value) {
                $this->settingService->modify($value['key'], $value['value']);
            }

            return response()->json(['success' => __('message.saved_successfully')]);
        }

        if (request()->has('site_logo')) {

            Cache::forget('settings');

            if (request()->hasFile('favicon')) {
                $getFileName      = pathinfo(request()->favicon->getClientOriginalName(), PATHINFO_FILENAME);
                $getFileExtension = pathinfo(request()->favicon->getClientOriginalExtension(), PATHINFO_FILENAME);

                $filename = $getFileName . '-' . Str::random(10) . '.' . $getFileExtension;

                $img = Image::make(request('favicon'));
                $resizeImage = $img->resize(32, 32);
                $img->insert($resizeImage, 'center');

                $img->save(storage_path('app/public/assets') . '/' . $filename);

                $this->settingService->modify('favicon', $filename);
            }

            if (request()->hasFile('logo_web_light')) {
                $logoWebLight = request('logo_web_light');
                $filename     = $logoWebLight->getClientOriginalName();

                $logoWebLight->storeAs('assets', $filename,$this->settingService->diskName());

                $this->settingService->modify('logo_web_light', $filename);
            }

            if (request()->hasFile('logo_web_dark')) {
                $logoWebDark = request('logo_web_dark');
                $filename    = $logoWebDark->getClientOriginalName();

                $logoWebDark->storeAs('assets', $filename, $this->settingService->diskName());

                $this->settingService->modify('logo_web_dark', $filename);
            }

            if (request()->hasFile('ogi_homepage')) {
                $ogi_homepage = request('ogi_homepage');
                $filename = $ogi_homepage->getClientOriginalName();

                $ogi_homepage->storeAs('assets', $filename, $this->settingService->diskName());

                $this->settingService->modify('ogi_homepage', $filename);
            }

            if (request()->hasFile('logo_dashboard')) {
                $logo_dashboard = request('logo_dashboard');
                $filename       = $logo_dashboard->getClientOriginalName();

                $logo_dashboard->storeAs('assets', $filename,$this->settingService->diskName());

                $this->settingService->modify('logo_dashboard', $filename);
            }

            if (request()->hasFile('logo_auth')) {
                $logo_auth = request('logo_auth');
                $filename  = $logo_auth->getClientOriginalName();

                $logo_auth->storeAs('assets', $filename,$this->settingService->diskName());

                $this->settingService->modify('logo_auth', $filename);
            }

            return Redirect::to(route('settings.index') . "#web-properties")->withSuccess(__('message.saved_successfully'));
        }

        if (request()->has('web_config')) {

            Cache::forget('settings');

            $arrayValues = [
                ['key' => 'measurement_id', 'value' => request('measurement_id')],
                ['key' => 'publisher_id', 'value' => request('publisher_id')],
                ['key' => 'property_id', 'value' => request('property_id')],
                ['key' => 'disqus_shortname', 'value' => request('disqus_shortname')],
            ];

            if (request()->hasFile('credentials_file')) {
                $filename = request()->credentials_file->getClientOriginalName();
                if (!File::exists(storage_path('app/analytics'))) {
                    File::makeDirectory(storage_path('app/analytics'));
                }
                request()->credentials_file->storeAs('', $filename, 'analytics');
                array_push($arrayValues, ['key' => 'credentials_file', 'value' => request('credentials_file')]);
            }

            if (!EnvEditor::keyExists("ANALYTICS_PROPERTY_ID")) {
                EnvEditor::addKey('ANALYTICS_PROPERTY_ID', request('property_id'));
            } else {
                EnvEditor::editKey('ANALYTICS_PROPERTY_ID', request('property_id'));
            }

            foreach ($arrayValues as $value) {
                $this->settingService->modify($value['key'], $value['value']);
            }

            return Redirect::to(route('settings.index') . "#web-config")->withSuccess(__('message.saved_successfully'));
        }

        if (request()->has('web_permalinks')) {

            Cache::forget('settings');

            if(request('permalink') == 'custom'){
                $permalink            = request('custom_input');
                $permalink_type       = request('permalink');
                $permalink_old_custom = request('custom_input');
            }else{
                $permalink            = request('permalink');
                $permalink_type       = request('permalink');
                $permalink_old_custom = config('settings.permalink_old_custom');
            }

            $page_permalinks = request('page_permalink_type');
            $category_permalinks = request('category_permalink_type');

            $arrayValues = [
                ['key' => 'permalink_type', 'value' => $permalink_type ],
                ['key' => 'permalink', 'value' => $permalink],
                ['key' => 'permalink_old_custom', 'value' => $permalink_old_custom],
                ['key' => 'page_permalink_type', 'value' => $page_permalinks],
                ['key' => 'category_permalink_type', 'value' => $category_permalinks],
            ];

            foreach ($arrayValues as $value) {
                $this->settingService->modify($value['key'], $value['value']);
            }

            return response()->json(['success' => __('message.saved_successfully')]);
        }
    }

    /**
     * @param Request $request
     * @return JsonResponse
     */
    public function createDataLink(Request $request)
    {
        $dataArr = json_decode(config('settings.links'), true);

        $recentLinks = Arr::last($dataArr);

        $newItem = [
            'id'    => $recentLinks ? $recentLinks['id'] + 1 : 1,
            'name'  => $request->name,
            'url'   => $request->url,
            'icon'  => $request->icon,
            'color' => $request->color
        ];

        array_push($dataArr, $newItem);

        Cache::forget('settings');
        $this->settingService->modify('links', json_encode($dataArr));

        return response()->json(['data' => $dataArr, 'success' => __('message.saved_successfully')]);
    }

    /**
     * @param $id
     * @return JsonResponse
     */
    public function removeDataLink($id)
    {
        $dataArr = json_decode(config('settings.links'), true);

        foreach ($dataArr as $key => $value) {
            if ($value['id'] == $id) {
                Arr::forget($dataArr, $key);
            }
        }

        Cache::forget('settings');
        $this->settingService->modify('links', json_encode($dataArr));

        return response()->json(['success' => __('message.deleted_successfully')]);
    }


    /**
     * @return Application|Factory|View|\Illuminate\Foundation\Application
     */
    public function openGraphImage()
    {
        $ogiCategory    = $this->settingService->getAssetImage(config('settings.ogi_category'));
        $ogiContact     = $this->settingService->getAssetImage(config('settings.ogi_contact'));
        $ogiPage        = $this->settingService->getAssetImage(config('settings.ogi_page'));
        $ogiPopularPost = $this->settingService->getAssetImage(config('settings.ogi_popular_post'));
        $ogiPosts       = $this->settingService->getAssetImage(config('settings.ogi_posts'));
        $ogiSearch      = $this->settingService->getAssetImage(config('settings.ogi_search'));
        $ogiArticlePost = $this->settingService->getAssetImage(config('settings.ogi_article_post'));
        $ogiVideoPost   = $this->settingService->getAssetImage(config('settings.ogi_video_post'));
        $ogiAudioPost   = $this->settingService->getAssetImage(config('settings.ogi_audio_post'));
        $ogiTag         = $this->settingService->getAssetImage(config('settings.ogi_tag'));

        return view('admin.settings.ogi-custom', compact(
            'ogiCategory', 'ogiContact', 'ogiPage', 'ogiPopularPost', 'ogiPosts', 'ogiSearch', 'ogiArticlePost', 'ogiVideoPost', 'ogiAudioPost', 'ogiTag'
        ));
    }

    /**
     * @return mixed
     */
    public function ogiCustomUpdate()
    {
        Cache::forget('settings');

        if (request()->hasFile('ogi_category')) {
            $file = request('ogi_category');
            $this->settingService->openGraphUpdate($file, 'category');
        }

        if (request()->hasFile('ogi_contact')) {
             $file = request('ogi_contact');
            $this->settingService->openGraphUpdate($file, 'contact');
        }

        if (request()->hasFile('ogi_page')) {
            $file = request('ogi_page');
            $this->settingService->openGraphUpdate($file, 'page');
        }

        if (request()->hasFile('ogi_popular_post')) {
            $file = request('ogi_popular_post');
            $this->settingService->openGraphUpdate($file, 'popular_post');
        }

        if (request()->hasFile('ogi_posts')) {
            $file = request('ogi_posts');
            $this->settingService->openGraphUpdate($file, 'posts');
        }

        if (request()->hasFile('ogi_search')) {
            $file = request('ogi_search');
            $this->settingService->openGraphUpdate($file, 'search');
        }

        if (request()->hasFile('ogi_article_post')) {
            $file = request('ogi_article_post');
            $this->settingService->openGraphUpdate($file, 'article_post');
        }

        if (request()->hasFile('ogi_video_post')) {
            $file = request('ogi_video_post');
            $this->settingService->openGraphUpdate($file, 'video_post');
        }

        if (request()->hasFile('ogi_audio_post')) {
            $file = request('ogi_audio_post');
            $this->settingService->openGraphUpdate($file, 'audio_post');
        }

        if (request()->hasFile('ogi_article_post')) {
            $file = request('ogi_article_post');
            $this->settingService->openGraphUpdate($file, 'article_post');
        }

        if (request()->hasFile('ogi_tag')) {
            $file = request('ogi_tag');
            $this->settingService->openGraphUpdate($file, 'tag');
        }

        return redirect()->route('ogi.index')
                ->withSuccess(__('message.saved_successfully'));
    }

    /**
     * @return JsonResponse
     */
    public function import()
    {
        $validator = Validator::make(request()->all(), [
            'import' => 'required|mimes:xlsx'
        ]);

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()]);
        }

        Excel::import(new MagzImport, request()->file('import'));

        return response()->json(['success' => __('setting.message_import_successfully')]);
    }

    /**
     * @return BinaryFileResponse|Response
     */
    public function export()
    {
        $name = config('retenvi.app_name') . '-v' . config('retenvi.version');
        $extension = '.xlsx';
        $fileName = $name . '.' . $extension;
        return (new MagzExport())->download($fileName);
    }

    /**
     * @return BinaryFileResponse
     */
    public function exportStorage()
    {
        $zip = new ZipArchive;

        $name      = config('retenvi.app_name') . '-v' . config('retenvi.version');
        $extension = 'storage.zip';
        $fileName  = $name . '-' . $extension;

        if ($zip->open(storage_path($fileName), ZipArchive::CREATE) === TRUE) {
            if ($this->diskStorage()->exists('analytics')) {
                $ads = File::files(storage_path('app/analytics'));
                foreach ($ads as $key => $value) {
                    $relativeNameInZipFile = 'analytics/' . basename($value);
                    $zip->addFile($value, $relativeNameInZipFile);
                }
            }

            if ($this->diskStorage()->exists('ad')) {
                $ads = File::files(storage_path('app/public/ad'));
                foreach ($ads as $key => $value) {
                    $relativeNameInZipFile = 'public/ad/' . basename($value);
                    $zip->addFile($value, $relativeNameInZipFile);
                }
            }

            if ($this->diskStorage()->exists('audios')) {
                $ads = File::files(storage_path('app/public/audios'));
                foreach ($ads as $key => $value) {
                    $relativeNameInZipFile = 'public/audios/' . basename($value);
                    $zip->addFile($value, $relativeNameInZipFile);
                }
            }

            if ($this->diskStorage()->exists('assets')) {
                $assets = File::files(storage_path('app/public/assets'));
                foreach ($assets as $key => $value) {
                    $relativeNameInZipFile = 'public/assets/' . basename($value);
                    $zip->addFile($value, $relativeNameInZipFile);
                }
            }

            if ($this->diskStorage()->exists('avatar')) {
                $avatars = File::files(storage_path('app/public/avatar'));
                foreach ($avatars as $key => $value) {
                    $relativeNameInZipFile = 'public/avatar/' . basename($value);
                    $zip->addFile($value, $relativeNameInZipFile);
                }
            }

            if ($this->diskStorage()->exists('file')) {
                $file_localize = File::files(storage_path('app/public/file'));
                foreach ($file_localize as $key => $value) {
                    $relativeNameInZipFile = 'public/file/' . basename($value);
                    $zip->addFile($value, $relativeNameInZipFile);
                }
            }

            if ($this->diskStorage()->exists('images')) {
                $images = File::files(storage_path('app/public/images'));
                foreach ($images as $key => $value) {
                    $relativeNameInZipFile = 'public/images/' . basename($value);
                    $zip->addFile($value, $relativeNameInZipFile);
                }
            }

            if ($this->diskStorage()->exists('videos')) {
                $images = File::files(storage_path('app/public/videos'));
                foreach ($images as $key => $value) {
                    $relativeNameInZipFile = 'public/videos/' . basename($value);
                    $zip->addFile($value, $relativeNameInZipFile);
                }
            }

            $zip->close();
        }

        return response()->download(storage_path($fileName))->deleteFileAfterSend(true);
    }
    
    /**
     * importFrom134
     *
     * @return void
     */
    public function importFrom134 ()
    {
        $file = $this->settingService->diskStorage()->get('file/db.json');

        if ($this->settingService->diskStorage()->missing('file/db.json')) {
            return response()->json(['error' => __('message.import_json_file_not_found')]);
        }

        $data = json_decode($file, true);

        foreach($data as $i => $v) {
            if ($v['type'] == 'header') {
                Arr::forget($data, $i);
            }

            if ($v['type'] == 'database') {
                Arr::forget($data, $i);
            }   
        }

        foreach ($data as $value) {
            $newArr[$value['name']] = $value['data'];
        }

        // get value setting
        function get($key, $settings)
        {
            $key = array_search($key, array_column($settings, 'key'));
            return $settings[$key]['value'];
        }

        // site socialmedia
        $jsonLinks = [];
        if (!empty($newArr['site_socialmedia'])) {
            $n = 0;
            foreach ($newArr['site_socialmedia'] as $ss) {
                $n++;
                foreach ($newArr['socialmedia'] as $sm) {
                    if ($ss['socialmedia_id'] == $sm['id']) {
                        $jsonLinks[] = [
                            'id'    => $n,
                            'name'  => $sm['name'],
                            'url'   => $ss['url'],
                            'icon'  => $sm['icon'],
                            'color' => $sm['color'],
                        ];
                    }
                }
            } 
        }

        
        // settings
        if ($newArr['settings']) {
            DB::statement('SET FOREIGN_KEY_CHECKS=0;');
            DB::table('settings')->truncate();
            DB::statement('SET FOREIGN_KEY_CHECKS=1;');

            if ($jsonLinks) {
                foreach ($jsonLinks as $i => $link) {
                    $__data[$i] = $link;
                }
            } else {
                $__data = [];
            }

            $_data = !empty($__data) ? $__data : [];

            DB::table('settings')->insert([
                ['group' => 'site_information', 'key' => 'company_name', 'value' => get('company_name', $newArr['settings'])],
                ['group' => 'site_information', 'key' => 'site_name', 'value' => get('site_name', $newArr['settings'])],
                ['group' => 'site_information', 'key' => 'site_url', 'value' => get('site_url', $newArr['settings'])],
                ['group' => 'site_information', 'key' => 'site_email', 'value' => get('site_email', $newArr['settings'])],
                ['group' => 'site_information', 'key' => 'site_phone', 'value' => get('site_phone', $newArr['settings'])],
                ['group' => 'site_information', 'key' => 'street', 'value' => get('street', $newArr['settings'])],
                ['group' => 'site_information', 'key' => 'city', 'value' => get('city', $newArr['settings'])],
                ['group' => 'site_information', 'key' => 'postal_code', 'value' => get('postal_code', $newArr['settings'])],
                ['group' => 'site_information', 'key' => 'state', 'value' => get('state', $newArr['settings'])],
                ['group' => 'site_information', 'key' => 'country', 'value' => get('country', $newArr['settings'])],
                ['group' => 'site_information', 'key' => 'site_description', 'value' => get('site_description', $newArr['settings'])],
                ['group' => 'site_information', 'key' => 'contact_description', 'value' => get('contact_description', $newArr['settings'])],
                ['group' => 'site_information', 'key' => 'meta_keyword', 'value' => get('meta_keyword', $newArr['settings'])],
                ['group' => 'site_information', 'key' => 'links', 'value' => json_encode($_data)],
                ['group' => 'site_information', 'key' => 'version', 'value' => '2.0.0'],
                ['group' => 'site_config', 'key' => 'maintenance', 'value' => get('maintenance', $newArr['settings'])],
                ['group' => 'site_config', 'key' => 'current_theme', 'value' => get('current_theme', $newArr['settings'])],
                ['group' => 'site_config', 'key' => 'current_theme_dir', 'value' => get('current_theme_dir', $newArr['settings'])],
                ['group' => 'site_config', 'key' => 'register', 'value' => get('register', $newArr['settings'])],
                ['group' => 'site_config', 'key' => 'email_verification', 'value' => get('email_verification', $newArr['settings'])],
                ['group' => 'site_config', 'key' => 'display_language', 'value' => get('display_language', $newArr['settings'])],
                ['group' => 'site_config', 'key' => 'default_language', 'value' => get('theme_language', $newArr['settings'])],
                ['group' => 'site_config', 'key' => 'credentials_file', 'value' => 'service-account-credentials.json'],
                ['group' => 'site_config', 'key' => 'disqus_shortname', 'value' => get('disqus_shortname', $newArr['settings'])],
                ['group' => 'site_config', 'key' => 'mailchimp', 'value' => get('mailchimp', $newArr['settings'])],
                ['group' => 'logo_image', 'key' => 'favicon', 'value' => get('favicon', $newArr['settings'])],
                ['group' => 'logo_image', 'key' => 'logo_web_light', 'value' => get('logo_web_light', $newArr['settings'])],
                ['group' => 'logo_image', 'key' => 'logo_web_dark', 'value' => get('logo_web_dark', $newArr['settings'])],
                ['group' => 'logo_image', 'key' => 'ogi_homepage', 'value' => get('og_image', $newArr['settings'])],
                ['group' => 'logo_image', 'key' => 'ogi_category', 'value' => get('ogi_category', $newArr['settings'])],
                ['group' => 'logo_image', 'key' => 'ogi_contact', 'value' => get('ogi_contact', $newArr['settings'])],
                ['group' => 'logo_image', 'key' => 'ogi_page', 'value' => get('ogi_page', $newArr['settings'])],
                ['group' => 'logo_image', 'key' => 'ogi_popular_post', 'value' => get('ogi_popular_post', $newArr['settings'])],
                ['group' => 'logo_image', 'key' => 'ogi_posts', 'value' => get('ogi_posts', $newArr['settings'])],
                ['group' => 'logo_image', 'key' => 'ogi_search', 'value' => get('ogi_search', $newArr['settings'])],
                ['group' => 'logo_image', 'key' => 'ogi_article_post', 'value' => get('ogi_article_post', $newArr['settings'])],
                ['group' => 'logo_image', 'key' => 'ogi_video_post', 'value' => get('ogi_video_post', $newArr['settings'])],
                ['group' => 'logo_image', 'key' => 'ogi_audio_post', 'value' => get('ogi_audio_post', $newArr['settings'])],
                ['group' => 'logo_image', 'key' => 'ogi_tag', 'value' => get('ogi_tag', $newArr['settings'])],
                ['group' => 'logo_image', 'key' => 'logo_dashboard', 'value' => get('logo_dashboard', $newArr['settings'])],
                ['group' => 'logo_image', 'key' => 'logo_auth', 'value' => get('logo_auth', $newArr['settings'])],
                ['group' => 'google', 'key' => 'measurement_id', 'value' => get('google_analytics_id', $newArr['settings'])],
                ['group' => 'google', 'key' => 'property_id', 'value' => ''],
                ['group' => 'google', 'key' => 'publisher_id', 'value' => get('publisher_id', $newArr['settings'])],
                ['group' => 'google', 'key' => 'google_site_verification', 'value' => get('google_site_verification', $newArr['settings'])],
                ['group' => 'permalinks', 'key' => 'permalink_type', 'value' => get('permalink_type', $newArr['settings'])],
                ['group' => 'permalinks', 'key' => 'permalink', 'value' => get('permalink', $newArr['settings'])],
                ['group' => 'permalinks', 'key' => 'permalink_old_custom', 'value' => get('permalink_old_custom', $newArr['settings'])],
                ['group' => 'permalinks', 'key' => 'page_permalink_type', 'value' => get('page_permalink_type', $newArr['settings'])],
                ['group' => 'permalinks', 'key' => 'category_permalink_type', 'value' => get('category_permalink_type', $newArr['settings'])],
            ]);
        }

        // languages
        if ($newArr['languages']) {
            DB::statement('SET FOREIGN_KEY_CHECKS=0;');
            DB::table('languages')->truncate();
            DB::statement('SET FOREIGN_KEY_CHECKS=1;');

            foreach ($newArr['languages'] as $language) {
                DB::table('languages')->insert([
                    'id' => $language['id'],
                    'name' => $language['language'],
                    'language' => $language['language_code'],
                    'country' => $language['country'],
                    'country_code' => $language['country_code'],
                    'direction' => $language['direction'],
                    'active' => $language['active'],
                    'created_at' => $language['created_at'],
                    'updated_at' => $language['updated_at']
                ]);
            }
        }

        // roles
        if ($newArr['roles']) {
            DB::statement('SET FOREIGN_KEY_CHECKS=0;');
            DB::table('roles')->truncate();
            DB::statement('SET FOREIGN_KEY_CHECKS=1;');

            foreach ($newArr['roles'] as $role) {
                DB::table('roles')->insert([
                    'id' => $role['id'],
                    'name' => $role['name'],
                    'guard_name' => $role['guard_name'],
                    'created_at' => $role['created_at'],
                    'updated_at' => $role['updated_at']
                ]);
            }
        }

        // role_has_permissions
        if ($newArr['role_has_permissions']) {
            DB::statement('SET FOREIGN_KEY_CHECKS=0;');
            DB::table('role_has_permissions')->truncate();
            DB::statement('SET FOREIGN_KEY_CHECKS=1;');

            foreach ($newArr['role_has_permissions'] as $rhp) {
                DB::table('role_has_permissions')->insert([
                    'permission_id' => $rhp['permission_id'],
                    'role_id' => $rhp['role_id']
                ]);
            }
        }


        // user links
        $jsonLinks = [];
        if (!empty($newArr['user_socialmedia'])) {
            foreach ($newArr['user_socialmedia'] as $us) {
                $n = 0;
                foreach ($newArr['socialmedia'] as $sm) {
                    $n++;
                    if ($us['socialmedia_id'] == $sm['id']) {
                        $jsonLinks[$us['user_id']][] = [
                            'id'    => $n,
                            'label' => $sm['name'],
                            'url'   => $us['url'],
                            'icon'  => $sm['icon'],
                            'color' => $sm['color'],
                        ];
                    }
                }
            }    
        }

        // users
        if ($newArr['users']) {
            DB::statement('SET FOREIGN_KEY_CHECKS=0;');
            DB::table('users')->truncate();
            DB::statement('SET FOREIGN_KEY_CHECKS=1;');

            foreach ($newArr['users'] as $user) {

                if ($jsonLinks) {
                    foreach ($jsonLinks as $i => $link) {
                        $__data[$i] = $link;
                    }
                } else {
                    $__data = [];
                }

                if (!empty($__data)) {
                    $_data = !empty($__data[$user['id']]) ? $__data[$user['id']] : [];
                } else {
                    $_data = [];
                }

                DB::table('users')->insert([
                    'id'         => $user['id'],
                    'name'       => $user['name'],
                    'email'      => $user['email'],
                    'username'   => $user['username'],
                    'password'   => $user['password'],
                    'created_at' => $user['created_at'],
                    'updated_at' => $user['updated_at'],
                    'photo'      => $user['photo'],
                    'about'      => $user['about'],
                    'links'      => json_encode($_data),
                    'occupation' => $user['occupation'],
                    'language'   => $user['language'],
                    'darkmode'   => $user['darkmode'],
                    'active'     => $user['active'],
                    'banned_at'  => $user['banned_at']
                ]);
            }
        }

        // terms
        if ($newArr['terms']) {
            DB::statement('SET FOREIGN_KEY_CHECKS=0;');
            DB::table('terms')->truncate();
            DB::statement('SET FOREIGN_KEY_CHECKS=1;');

            foreach ($newArr['terms'] as $term) {
                DB::table('terms')->insert([
                    'id' => $term['id'],
                    'name' => $term['name'],
                    'slug' => $term['slug'],
                    'taxonomy' => $term['taxonomy'],
                    'description' => $term['description'],
                    'image' => $term['image'],
                    'parent' => $term['parent'],
                    'language_id' => $term['language_id'],
                    'translation' => $term['translation'],
                    'created_at' => $term['created_at'],
                    'updated_at' => $term['updated_at']
                ]);
            }
        }

        // posts
        if ($newArr['posts']) {
            DB::statement('SET FOREIGN_KEY_CHECKS=0;');
            DB::table('posts')->truncate();
            DB::statement('SET FOREIGN_KEY_CHECKS=1;');

            foreach ($newArr['posts'] as $post) {
                DB::table('posts')->insert([
                    'id' => $post['id'],
                    'post_title' => $post['post_title'],
                    'post_name' => $post['post_name'],
                    'post_summary' => $post['post_summary'],
                    'post_content' => $post['post_content'],
                    'meta_description' => $post['meta_description'],
                    'meta_keyword' => $post['meta_keyword'],
                    'post_status' => $post['post_status'],
                    'post_visibility' => $post['post_visibility'],
                    'post_author' => $post['post_author'],
                    'post_language' => $post['post_language'],
                    'post_type' => $post['post_type'],
                    'post_guid' => $post['post_guid'],
                    'post_hits' => $post['post_hits'],
                    'like' => $post['like'],
                    'post_image' => $post['post_image'],
                    'post_image_meta' => $post['post_image_meta'],
                    'post_mime_type' => $post['post_mime_type'],
                    'comment_status' => $post['comment_status'],
                    'comment_count' => $post['comment_count'],
                    'created_at' => $term['created_at'],
                    'updated_at' => $term['updated_at']
                ]);
            }
        }

        // post term
        if ($newArr['post_term']) {
            DB::statement('SET FOREIGN_KEY_CHECKS=0;');
            DB::table('post_term')->truncate();
            DB::statement('SET FOREIGN_KEY_CHECKS=1;');

            foreach ($newArr['post_term'] as $postterm) {
                DB::table('post_term')->insert([
                    'post_id' => $postterm['post_id'],
                    'term_id' => $postterm['term_id'],
                    'created_at' => $postterm['created_at'],
                    'updated_at' => $postterm['updated_at']
                ]);
            }
        }

        // translations
        if ($newArr['translations']) {
            DB::statement('SET FOREIGN_KEY_CHECKS=0;');
            DB::table('translations')->truncate();
            DB::statement('SET FOREIGN_KEY_CHECKS=1;');

            foreach ($newArr['translations'] as $translation) {
                DB::table('translations')->insert([
                    'id' => $translation['id'],
                    'value' => $translation['value']
                ]);
            }
        }

        // post translations
        if ($newArr['post_translations']) {
            DB::statement('SET FOREIGN_KEY_CHECKS=0;');
            DB::table('post_translations')->truncate();
            DB::statement('SET FOREIGN_KEY_CHECKS=1;');

            foreach ($newArr['post_translations'] as $postTrans) {
                DB::table('post_translations')->insert([
                    'post_id' => $postTrans['post_id'],
                    'translation_id' => $postTrans['translation_id']
                ]);
            }
        }

        // menus
        if ($newArr['menus']) {
            DB::statement('SET FOREIGN_KEY_CHECKS=0;');
            DB::table('menus')->truncate();
            DB::statement('SET FOREIGN_KEY_CHECKS=1;');

            foreach ($newArr['menus'] as $menu) {
                DB::table('menus')->insert([
                    'id' => $menu['id'],
                    'name' => $menu['name'],
                    'created_at' => $menu['created_at'],
                    'updated_at' => $menu['updated_at']
                ]);
            }
        }

        // menu item
        if ($newArr['menu_items']) {
            DB::statement('SET FOREIGN_KEY_CHECKS=0;');
            DB::table('menu_items')->truncate();
            DB::statement('SET FOREIGN_KEY_CHECKS=1;');

             foreach ($newArr['menu_items'] as $item) {
                DB::table('menu_items')->insert([
                    'id' => $item['id'],
                    'label' => $item['label'],
                    'link' => $item['link'],
                    'parent' => $item['parent'],
                    'sort' => $item['sort'],
                    'class' => $item['class'],
                    'menu_id' => $item['menu_id'],
                    'language' => $item['language'],
                    'depth' => $item['depth'],
                    'created_at' => $item['created_at'],
                    'updated_at' => $item['updated_at']
                ]);
             }
        }

        // menu_languages
        if ($newArr['menu_languages']) {
            DB::statement('SET FOREIGN_KEY_CHECKS=0;');
            DB::table('menu_languages')->truncate();
            DB::statement('SET FOREIGN_KEY_CHECKS=1;');

            foreach ($newArr['menu_languages'] as $menuLang) {
                DB::table('menu_languages')->insert([
                    'id' => $menuLang['id'],
                    'menu_id' => $menuLang['menu_id'],
                    'language_id' => $menuLang['language_id'],
                    'created_at' => $menuLang['created_at'],
                    'updated_at' => $menuLang['updated_at']
                ]);
            }
        }

        // bans
        if ($newArr['bans']) {
            DB::statement('SET FOREIGN_KEY_CHECKS=0;');
            DB::table('bans')->truncate();
            DB::statement('SET FOREIGN_KEY_CHECKS=1;');

            foreach ($newArr['bans'] as $ban) {
                DB::table('bans')->insert([
                    'id' => $ban['id'],
                    'bannable_type' => $ban['bannable_type'],
                    'bannable_id' => $ban['bannable_id'],
                    'created_by_type' => $ban['created_by_type'],
                    'created_by_id' => $ban['created_by_id'],
                    'comment' => $ban['comment'],
                    'expired_at' => $ban['expired_at'],
                    'deleted_at' => $ban['deleted_at'],
                    'created_at' => $ban['created_at'],
                    'updated_at' => $ban['updated_at']
                ]);
            }
        }

        // contacts
        if ($newArr['contacts']) {
            DB::statement('SET FOREIGN_KEY_CHECKS=0;');
            DB::table('contacts')->truncate();
            DB::statement('SET FOREIGN_KEY_CHECKS=1;');

            foreach ($newArr['contacts'] as $contact) {
                DB::table('contacts')->insert([
                    'id' => $contact['id'],
                    'name' => $contact['name'],
                    'email' => $contact['email'],
                    'subject' => $contact['subject'],
                    'message' => $contact['message'],
                    'status' => $contact['status'],
                    'created_at' => $contact['created_at'],
                    'updated_at' => $contact['updated_at']
                ]);
            }
        }

        // model_has_roles
        if ($newArr['model_has_roles']) {
            DB::statement('SET FOREIGN_KEY_CHECKS=0;');
            DB::table('model_has_roles')->truncate();
            DB::statement('SET FOREIGN_KEY_CHECKS=1;');

            foreach ($newArr['model_has_roles'] as $model) {
                DB::table('model_has_roles')->insert([
                    'role_id' => $model['role_id'],
                    'model_type' => $model['model_type'],
                    'model_id' => $model['model_id']
                ]);
            }
        }

        return response()->json(['success' => __('message.import_successfully')]);
    }
}
